'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _utils = require('./utils');

var _item = require('./item.js');

var _group = require('./group.js');

var _strip = require('./strip.js');

var _column = require('./column.js');

var _layoutsStore = require('./layoutsStore.js');

var _layoutsStore2 = _interopRequireDefault(_layoutsStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Layouter = function () {
  function Layouter(layoutParams) {
    (0, _classCallCheck3.default)(this, Layouter);


    this.ready = false;
    this.pointer = 0;
    this.layoutItems = [];

    this.findNeighborItem = this.findNeighborItem.bind(this);

    this.updateParams(layoutParams);

    if (this.createLayoutOnInit !== false) {
      this.createLayout(layoutParams);
    }
  }

  (0, _createClass3.default)(Layouter, [{
    key: 'updateParams',
    value: function updateParams(layoutParams) {
      this.srcItems = layoutParams.items;
      this.styleParams = _utils.utils.convertStyleParams(layoutParams.styleParams);
      this.container = _utils.utils.convertContainer(layoutParams.container, this.styleParams);

      var options = layoutParams.options || {};
      this.useExistingLayout = !!options.useExistingLayout;
      this.createLayoutOnInit = options.createLayoutOnInit;
      this.showAllItems = !!options.showAllItems || !!layoutParams.showAllItems;
      this.skipVisibilitiesCalc = !!options.skipVisibilitiesCalc;
      this.useLayoutStore = !!options.useLayoutStore;
    }
  }, {
    key: 'verifyGalleryState',
    value: function verifyGalleryState() {
      if (!this.container.galleryWidth) {
        this.ready = false;
        this.reason = 'galleryWidth is undefined or 0';
        return false;
      }

      if (!this.styleParams.gallerySize) {
        this.ready = false;
        this.reason = 'gallerySize is undefined or 0';
        return false;
      }

      return true;
    }
  }, {
    key: 'calcNumberOfColumns',
    value: function calcNumberOfColumns(galleryWidth, gallerySize) {
      var numOfCols = 1;
      if (this.styleParams.isVertical) {
        if (this.styleParams.fixedColumns > 0) {
          numOfCols = this.styleParams.fixedColumns;
        } else if (this.styleParams.columnWidths) {
          numOfCols = this.styleParams.columnWidths.split(',').length;
        } else {
          numOfCols = Math.ceil(galleryWidth / gallerySize) || 1;
        }
      } else {
        numOfCols = 1;
      }
      return numOfCols;
    }
  }, {
    key: 'findShortestColumn',
    value: function findShortestColumn(columns, groupIdx) {
      var minCol = columns[0];
      if (this.styleParams.placeGroupsLtr) {
        minCol = columns[groupIdx % columns.length];
      } else {
        var minColH = -1;
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = (0, _getIterator3.default)(columns), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var column = _step.value;

            var colH = column.height;
            if (colH < minColH || minColH < 0) {
              minColH = colH;
              minCol = column;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
      return minCol;
    }
  }, {
    key: 'saveExistingLayout',
    value: function saveExistingLayout() {
      if (this.useLayoutStore) {
        _layoutsStore2.default.layout = {
          pointer: this.pointer,
          layoutItems: this.layoutItems,
          groups: this.groups,
          strips: this.strips,
          groupIdx: this.groupIdx,
          groupItems: this.groupItems,
          group: this.group,
          strip: this.strip,
          gallerySize: this.gallerySize,
          galleryHeight: this.galleryHeight,
          columns: this.columns
        };
      }
    }
  }, {
    key: 'prepareLayoutParams',
    value: function prepareLayoutParams() {
      var _this = this;

      if (this.useExistingLayout && this.pointer > 0) {

        if (this.useLayoutStore) {
          (0, _assign2.default)(this, _layoutsStore2.default.layout);
        } else {

          if (this.styleParams.isVertical) {

            //---------------------| COLUMNS GALLERY |----------------------//
            //remove items from the last 3 groups;
            var lastGroups = this.groups.slice(-3);
            lastGroups.forEach(function (group) {
              var column = _this.columns[group.columnIdx];
              if (column) {
                column.height -= group.totalHeight;
                column.groups.splice(-1, 1);
              }
              _this.groups.splice(-1, 1);
              group.items.forEach(function () {
                _this.layoutItems.splice(-1, 1);
                _this.pointer--;
              });
              _this.groupIdx--;
            });
          } else {

            //---------------------| STRIPS GALLERY |----------------------//
            var lastStrip = this.strips[this.strips.length - 1];
            if (lastStrip) {
              this.strips.splice(-1, 1);
              var groups = lastStrip.groups;
              groups.forEach(function (group) {
                _this.groups.splice(-1, 1);
                group.items.forEach(function () {
                  _this.layoutItems.splice(-1, 1);
                  _this.pointer--;
                });
                _this.groupIdx--;
              });
              this.galleryHeight = this.strips.reduce(function (totalHeight, strip) {
                return totalHeight += strip.height;
              }, 0);
              // this.strip = this.strips[this.strips.length - 1];
              this.strip = new _strip.Strip({
                idx: this.strips.length,
                container: this.container,
                styleParams: this.styleParams
              });
            }
          }

          this.groupItems = [];
        }

        this.item = {};
        this.maxLoops = this.srcItems.length * 10;
      } else {

        this.pointer = 0;
        this.firstGroup = false;
        this.layoutItems = [];
        this.groups = [];
        this.strips = [];

        var gallerySizeVal = typeof this.styleParams.gallerySize === 'function' ? this.styleParams.gallerySize() : this.styleParams.gallerySize;
        this.gallerySize = Math.floor(gallerySizeVal) + Math.ceil(2 * (this.styleParams.imageMargin - this.styleParams.galleryMargin));
        this.galleryWidth = Math.floor(this.container.galleryWidth);
        this.maxGroupSize = this.getMaxGroupSize();

        this.groupIdx = 0;
        this.item = {};

        this.groupItems = [];
        this.group;
        this.bounds = this.container.bounds || {};

        this.strip = new _strip.Strip({
          idx: 1,
          container: this.container,
          styleParams: this.styleParams
        });

        this.galleryHeight = 0;

        this.numOfCols = this.calcNumberOfColumns(this.galleryWidth, this.gallerySize);
        this.gallerySize = this.styleParams.isVertical ? Math.floor(this.galleryWidth / this.numOfCols) : this.gallerySize;

        var _styleParams = this.styleParams,
            columnWidths = _styleParams.columnWidths,
            cubeRatio = _styleParams.cubeRatio;


        var columnWidthsArr = columnWidths && columnWidths.length > 0 ? columnWidths.split(',') : false;
        this.columns = Array(this.numOfCols).fill(0).map(function (column, idx) {
          return new _column.Column(idx, columnWidthsArr ? columnWidthsArr[idx] : _this.gallerySize, _this.styleParams.cubeRatio);
        });
        this.columns[this.numOfCols - 1].width += this.galleryWidth - this.columns.reduce(function (sum, col) {
          return col.width + sum;
        }, 0); //the last group compensates for half pixels in other groups
        this.columns[this.numOfCols - 1].cubeRatio = cubeRatio * (this.columns[this.numOfCols - 1].width / this.gallerySize); //fix the last group's cube ratio

        this.maxLoops = this.srcItems.length * 10;
      }
    }
  }, {
    key: 'createLayout',
    value: function createLayout(layoutParams) {

      if (typeof layoutParams !== 'undefined') {
        this.updateParams(layoutParams);
      }

      if (!this.verifyGalleryState()) {
        return false;
      }

      this.prepareLayoutParams();

      while (this.srcItems[this.pointer]) {

        if (this.imagesLeft === 6) {
          this.saveExistingLayout();
        }

        this.maxLoops--;
        if (this.maxLoops <= 0) {
          console.error('Cannot create layout, maxLoops reached!!!');
          return false;
        }

        this.item = new _item.Item({
          idx: this.pointer,
          inGroupIdx: this.groupItems.length + 1,
          scrollTop: this.galleryHeight,
          dto: this.srcItems[this.pointer],
          container: this.container,
          styleParams: this.styleParams
        });

        this.layoutItems[this.pointer] = this.item;

        //push the image to a group - until its full
        this.groupItems.push(this.item);
        if (this.groupItems.length < this.maxGroupSize && this.srcItems[this.pointer + 1]) {
          this.pointer++;
          continue;
        }

        this.group = new _group.Group({
          idx: this.groupIdx,
          stripIdx: this.strip.idx,
          inStripIdx: this.strip.groups.length + 1,
          top: this.galleryHeight,
          items: this.groupItems,
          isLastItems: this.isLastImages,
          gallerySize: this.gallerySize,
          showAllItems: this.showAllItems,
          container: this.container,
          styleParams: this.styleParams
        });
        this.groups[this.groupIdx] = this.group;

        //take back the pointer in case the group was created with less items
        this.pointer += this.group.realItems.length - this.groupItems.length;

        this.groupIdx++;
        this.groupItems = [];

        //resize and fit the group in the strip / column
        if (!this.styleParams.isVertical) {

          //---------------------| STRIPS GALLERY |----------------------//

          if (this.strip.isFull(this.group, this.isLastImage)) {
            //close the current strip
            this.strip.resizeToHeight(this.galleryWidth / this.strip.ratio);
            this.strip.setWidth(this.galleryWidth);
            this.galleryHeight += this.strip.height;
            this.columns[0].addGroups(this.strip.groups);
            this.strips.push(this.strip);

            //open a new strip
            this.strip = new _strip.Strip({
              idx: this.strip.idx + 1,
              container: this.container,
              styleParams: this.styleParams
            });

            //reset the group (this group will be rebuilt)
            this.pointer -= this.group.realItems.length - 1;
            this.groupIdx--;
            continue;
          }

          //add the group to the (current/new) strip
          this.group.setTop(this.galleryHeight);
          this.strip.ratio += this.group.ratio;
          // this.strip.height = Math.min(gallerySize, (galleryWidth / this.strip.ratio));
          this.strip.height = this.galleryWidth / this.strip.ratio;
          this.strip.setWidth(this.galleryWidth);
          this.strip.addGroup(this.group);

          if (this.isLastImage && this.strip.hasGroups) {
            if (this.styleParams.oneRow) {
              this.strip.height = this.container.galleryHeight + (this.styleParams.imageMargin - this.styleParams.galleryMargin);
            } else if (this.gallerySize * 1.5 < this.strip.height) {
              //stretching the this.strip to the full width will make it too high - so make it as high as the gallerySize and not stretch
              this.strip.height = this.gallerySize;
              this.strip.markAsIncomplete();
            }

            this.strip.resizeToHeight(this.strip.height);
            this.galleryHeight += this.strip.height;
            this.columns[0].addGroups(this.strip.groups);
            this.strips.push(this.strip);
          }
        } else {

          //---------------------| COLUMNS GALLERY |----------------------//

          //find the shortest column
          var minCol = this.findShortestColumn(this.columns, this.groups.length - 1);

          //resize the group and images
          this.group.fixItemsRatio(minCol.cubeRatio); //fix last column's items ratio (caused by stretching it to fill the screen)
          this.group.resizeToWidth(minCol.width);
          this.group.round();

          //update the group's position
          this.group.setTop(minCol.height);
          this.group.setLeft(minCol.idx * this.gallerySize);

          //add the image to the column
          minCol.addGroup(this.group);

          //add the image height to the column
          minCol.height += this.group.totalHeight;

          if (this.galleryHeight < minCol.height) {
            this.galleryHeight = minCol.height;
          }
        }

        //set the group visibility

        if (!this.skipVisibilitiesCalc && !this.gotScrollEvent && this.pointer < 10) {
          //until the first scroll event, make sure the first 10 groups are displayed
          this.group.calcVisibilities(true);
        } else {
          this.group.calcVisibilities(this.bounds);
        }

        if (!this.firstGroup) {
          this.firstGroup = this.group;
        }

        this.pointer++;
      }

      //results
      this.lastGroup = this.group;
      this.colWidth = Math.floor(this.galleryWidth / this.numOfCols);
      this.height = this.galleryHeight - (this.styleParams.imageMargin - this.styleParams.galleryMargin) * 2;

      if (!this.skipVisibilitiesCalc) {
        this.calcVisibilities(this.bounds);
      }

      this.ready = true;

      return this.scheme;
    }
  }, {
    key: 'calcVisibilities',
    value: function calcVisibilities(bounds) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(this.columns), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var column = _step2.value;
          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = (0, _getIterator3.default)(column.groups), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              var group = _step3.value;

              group.calcVisibilities(bounds);
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return this.scheme;
    }
  }, {
    key: 'lastVisibleItemIdxInHeight',
    value: function lastVisibleItemIdxInHeight(height) {
      for (var i = this.items.length - 1; i >= 0; i--) {
        var item = this.items[i];
        var isVisible = item.offset.top < height;
        if (isVisible) {
          return i;
        }
      }
      return this.items.length - 1;
    }
  }, {
    key: 'lastVisibleItemIdx',
    value: function lastVisibleItemIdx() {
      //the item must be visible and about the show more button
      return this.lastVisibleItemIdxInHeight(this.container.galleryHeight - 100);
    }
  }, {
    key: 'findNeighborItem',
    value: function findNeighborItem(itemIdx, dir) {
      var _this2 = this;

      var currentItem = this.layoutItems[itemIdx];

      var neighborItem = void 0;

      var findClosestItem = function findClosestItem(currentItemX, currentItemY, condition) {

        var minDistance = null;
        var minDistanceItem = {};

        var itemY = void 0;
        var itemX = void 0;
        var distance = void 0;

        // each(slice(this.layoutItems, itemIdx - 50, itemIdx + 50), (item) => {
        _this2.layoutItems.forEach(function (item) {
          itemY = item.offset.top + item.height / 2;
          itemX = item.offset.left + item.width / 2;
          distance = Math.sqrt(Math.pow(itemY - currentItemY, 2) + Math.pow(itemX - currentItemX, 2));
          if ((minDistance === null || distance > 0 && distance < minDistance) && condition(currentItemX, currentItemY, itemX, itemY)) {
            minDistance = distance;
            minDistanceItem = item;
          }
        });
        return minDistanceItem;
      };

      switch (dir) {
        case 'up':
          neighborItem = findClosestItem(currentItem.offset.left + currentItem.width / 2, currentItem.offset.top, function (curX, curY, itmX, itmY) {
            return itmY < curY;
          });
          break;

        case 'left':
          neighborItem = findClosestItem(currentItem.offset.left, currentItem.offset.top + currentItem.height / 2, function (curX, curY, itmX) {
            return itmX < curX;
          });
          break;

        case 'down':
          neighborItem = findClosestItem(currentItem.offset.left + currentItem.width / 2, currentItem.offset.bottom, function (curX, curY, itmX, itmY) {
            return itmY > curY;
          });
          break;

        default:
        case 'right':
          neighborItem = findClosestItem(currentItem.offset.right, currentItem.offset.top + currentItem.height / 2, function (curX, curY, itmX) {
            return itmX > curX;
          });
          break;

      }

      if (neighborItem.idx >= 0) {
        return neighborItem.idx;
      } else {
        console.warn('Could not find offset for itemIdx', itemIdx, dir);
        return itemIdx; //stay on the same item
      }
    }
  }, {
    key: 'getMaxGroupSize',
    value: function getMaxGroupSize() {
      var _maxGroupSize = 1;
      try {
        var groupTypes = typeof this.styleParams.groupTypes === 'string' && this.styleParams.groupTypes.length > 0 ? this.styleParams.groupTypes.split(',') : this.styleParams.groupTypes;
        _maxGroupSize = groupTypes.reduce(function (curSize, groupType) {
          return Math.max(curSize, parseInt(groupType));
        }, 1);
        _maxGroupSize = Math.min(_maxGroupSize, this.styleParams.groupSize);
      } catch (e) {
        console.error('couldn\'t calculate max group size - returing 3 (?)', e);
        _maxGroupSize = 3;
      }
      return _maxGroupSize;
    }
  }, {
    key: 'isLastImage',
    get: function get() {
      return !this.srcItems[this.pointer + 1];
    }
  }, {
    key: 'isLastImages',
    get: function get() {
      if (this.styleParams.layoutsVersion > 1) {
        //layouts version 2+
        return !this.srcItems[this.pointer + 1];
      } else {
        //Backwards compatibility
        return !this.srcItems[this.pointer + 3];
      }
    }
  }, {
    key: 'imagesLeft',
    get: function get() {
      return this.srcItems.length - this.pointer - 1;
    }
  }, {
    key: 'items',
    get: function get() {
      return this.layoutItems;
    }
  }, {
    key: 'scheme',
    get: function get() {
      return {
        items: this.items.map(function (item) {
          return item.scheme;
        }),
        groups: this.groups.map(function (group) {
          return group.scheme;
        }),
        strips: this.strips.map(function (strip) {
          return strip.scheme;
        }),
        columns: this.columns.map(function (column) {
          return column.scheme;
        }),
        height: this.height
      };
    }
  }]);
  return Layouter;
}();

exports.default = Layouter;